/*
 * =====================================================================================
 *
 *       Filename:  debug.c
 *
 *    Description:  Debugging functions
 *
 *        Version:  1.0
 *        Created:  27/04/2017 12:58:37
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Jerome Benoit (fraggle), jerome.benoit@piment-noir.org
 *   Organization:  Piment Noir
 *
 * =====================================================================================
 */

#include <string.h>

#include "othello.h"
#include "debug.h"

#if BUILD_TYPE == debug
void dbg_mvprintw(int y, int x, const char* debug_text, ...) {
    va_list func_args;
    char debug_msg[256];

    snprintf(debug_msg, sizeof(debug_msg), "%s%s", "DEBUG: ", debug_text);
    va_start(func_args, debug_text);
    mvprintw(y, x, debug_msg, func_args);
    va_end(func_args);
}
#else
void dbg_mvprintw(int y, int x, const char* debug_text, ...) { }
#endif

void display_array(int base_y, int base_x, unsigned int pawn_array[board_size][board_size]) {

    for (int i = 1; i <= board_size; i++) {
        dbg_mvprintw(base_y + i, base_x, "%d", i);
        for (int j = 1; j <= board_size; j++) {
            dbg_mvprintw(base_y, base_x + j, "%d", j);
            dbg_mvprintw(base_y + i, base_x + j, "%d", get_box_value(i, j, pawn_array));
        }
    }
}
